"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var transformation_type_enum_1 = require("./transformation-type.enum");
var direction_enum_1 = require("./direction.enum");
var CoordinatesService = (function () {
    function CoordinatesService() {
    }
    CoordinatesService.prototype.transform = function (value, transformationType, direction) {
        if (!transformationType || !this.isValueValid(value, direction)) {
            return '';
        }
        else if (transformationType === transformation_type_enum_1.TransformationType.ToDegrees) {
            return this.transformToDegrees(value, direction);
        }
        else {
            return this.transformToDigit(value);
        }
    };
    CoordinatesService.prototype.isValueValid = function (value, direction) {
        return (!value && value === 0) || this.isValidDigit(value, direction) || this.isValidDegree(value);
    };
    CoordinatesService.prototype.transformToDigit = function (value) {
        var potentialNumber = Number(value);
        if (!Number.isNaN(potentialNumber)) {
            return potentialNumber;
        }
        return this.transformDegreesToNumber(value);
    };
    CoordinatesService.prototype.transformToDegrees = function (value, direction) {
        if (typeof value === 'string' && Number.isNaN(Number(value))) {
            return value;
        }
        else {
            var numberValue = Number(value);
            return this.transformNumberToDegrees(numberValue) +
                (direction ? " " + this.resolveDirection(numberValue, direction) : '');
        }
    };
    CoordinatesService.prototype.isValidDegree = function (value, direction) {
        if (typeof value !== 'string' || !this.isValidDegreeFormat(value)) {
            return false;
        }
        var values = this.extractValuesFromDegrees(value);
        var inBoundary = function (val) { return val >= 0 && val < 60; };
        if (!inBoundary(values[1]) || !inBoundary(values[2])) {
            return false;
        }
        var boundaries = this.getDirectionBoundaries(direction);
        var digitValue = this.sumDegreeValues(values);
        return !(digitValue < boundaries[0] || digitValue > boundaries[1]);
    };
    CoordinatesService.prototype.isValidDigit = function (value, direction) {
        var val;
        if (typeof value === 'number' && !Number.isNaN(value)) {
            val = value;
        }
        else if (typeof value === 'string') {
            val = Number(value);
            if (Number.isNaN(val)) {
                return false;
            }
        }
        else {
            return false;
        }
        var boundaries = this.getDirectionBoundaries(direction);
        return !(value < boundaries[0] || value > boundaries[1]);
    };
    CoordinatesService.prototype.isValidDegreeFormat = function (value) {
        return /^\d+°(\d+'(\d+")?)?(\s[NSWE])?$/.test(value);
    };
    CoordinatesService.prototype.transformNumberToDegrees = function (value) {
        var absoluteDegrees = Math.abs(Number(value));
        var fullDegrees = Math.floor(absoluteDegrees);
        var remainingMinutes = (absoluteDegrees - fullDegrees) * 60;
        var minutes = Math.floor(remainingMinutes);
        var remainingSeconds = (remainingMinutes - minutes) * 60;
        var seconds = Math.round(remainingSeconds);
        if (seconds === 60) {
            minutes += 1;
            seconds = 0;
        }
        if (minutes === 60) {
            fullDegrees += 1;
            minutes = 0;
        }
        return (fullDegrees + "\u00B0" + minutes + "'" + seconds + "\"");
    };
    CoordinatesService.prototype.transformDegreesToNumber = function (value) {
        return this.sumDegreeValues(this.extractValuesFromDegrees(value)) * (this.isMinusHemisphere(value) ? -1 : 1);
    };
    CoordinatesService.prototype.isMinusHemisphere = function (value) {
        return /[SW]$/.test(value);
    };
    CoordinatesService.prototype.sumDegreeValues = function (values) {
        return values[0] + values[1] / 60 + values[2] / (60 * 60);
    };
    CoordinatesService.prototype.resolveDirection = function (degrees, direction) {
        if (direction === direction_enum_1.Direction.Latitude) {
            return degrees < 0 ? 'S' : 'N';
        }
        else {
            return degrees < 0 ? 'W' : 'E';
        }
    };
    CoordinatesService.prototype.extractValuesFromDegrees = function (value) {
        var values = value.split(/\D/).filter(function (part) { return Boolean(part); }).map(function (v) { return Number(v); });
        return values.concat(Array(3 - values.length).fill(0));
    };
    CoordinatesService.prototype.getDirectionBoundaries = function (direction) {
        return !direction || direction === direction_enum_1.Direction.Longitude ? [-180, 180] : [-90, 90];
    };
    return CoordinatesService;
}());
exports.CoordinatesService = CoordinatesService;
//# sourceMappingURL=coordinates.service.js.map